#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/resource.h>

#define N_ELEM 10000000

typedef struct {
  int x, y, z;
} int_vec3;

typedef struct {
  float x, y, z;
} float_vec3;

#define VECT3_SUM(_c, _a, _b) {                 \
    (_c).x = (_a).x + (_b).x;			\
    (_c).y = (_a).y + (_b).y;			\
    (_c).z = (_a).z + (_b).z;			\
  }

void test_int() {
  int_vec3 a[N_ELEM], b[N_ELEM], c[N_ELEM];
  unsigned int i;
  struct timeval t1, t2, t_diff;

  for (i = 0; i < N_ELEM; i++) {
    a[i].x = rand();
    a[i].y = rand();
    a[i].z = rand();
    b[i].x = rand();
    b[i].y = rand();
    b[i].z = rand();
  }

  gettimeofday(&t1, NULL);
  for (i = 0; i < N_ELEM; i++)
    VECT3_SUM(c[i], b[i], a[i]);
  gettimeofday(&t2, NULL);

  timersub(&t2, &t1, &t_diff);
  printf("Time with integers: %ld\n", t_diff.tv_sec*1000000 + t_diff.tv_usec);
}

void test_float() {
  float_vec3 a[N_ELEM], b[N_ELEM], c[N_ELEM];
  unsigned int i;
  struct timeval t1, t2, t_diff;

  for (i = 0; i < N_ELEM; i++) {
    a[i].x = ((float)rand())/RAND_MAX;
    a[i].y = ((float)rand())/RAND_MAX;
    a[i].z = ((float)rand())/RAND_MAX;
    b[i].x = ((float)rand())/RAND_MAX;
    b[i].y = ((float)rand())/RAND_MAX;
    b[i].z = ((float)rand())/RAND_MAX;
  }

  gettimeofday(&t1, NULL);
  for (i = 0; i < N_ELEM; i++)
    VECT3_SUM(c[i], b[i], a[i]);
  gettimeofday(&t2, NULL);

  timersub(&t2, &t1, &t_diff);
  printf("Time with floats: %ld\n", t_diff.tv_sec*1000000 + t_diff.tv_usec);
}

#define STACK_SIZE 1000000000

int main() {
  struct rlimit rl;
  int res;

  srand(0);

  res = getrlimit(RLIMIT_STACK, &rl);
  if (res != 0) {
    printf("Error getting stack size! (%d)\n", res);
    exit(1);
  }

  rl.rlim_cur = STACK_SIZE;
  res = setrlimit(RLIMIT_STACK, &rl);
  if (res != 0) {
    printf("Error setting stack size! (%d)\n", res);
    exit(1);
  }

  test_int();
  test_float();
}
